@page {
    size: A4;
    margin: 0.5cm;
    @bottom-right {
        content: "Page " counter(page) " of " counter(pages);
        font-size:50% !important;
    }
}

html {
    background-color:#FFFFFF;
}
html, body {
    margin: 0;
    padding: 0;
}

h1 { font-size:1.4em; margin:10px 0 10px 0; }
h2 { font-size:1.2em; }

.label {
    font-weight:bold;
}
.units {
    color:#777777;
    font-weight:normal;
    font-style:italic;
    font-size:10px;
}

table tr td,
table tr th {
    text-align:left;
}

table#section-header{
    border-bottom:1px solid #000000;
}

table#section-header,
table#section-info {
    width:100%;
    margin-bottom:0;
}
table#section-header tr td,
table#section-info tr td {
    vertical-align:top;
}
table#section-header td#lab-logo {

}
table#section-header td#accreditation-logo {
    width:35mm;
    margin-right:1cm;
    padding-right:1cm;
}

#section-alert {
    padding:10px 20px 10px 20px;
    border:1px solid #CDCDCD;
    background-color:#EFEFEF;
    margin:0;
}
#section-alert h1 {
    font-size:1em;
    margin-top:0;
    padding-bottom:0;
    border:none;
    color:#d40000;
}
#section-summary table,
#section-results table,
#section-qcresults table {
    padding-left:10px;
    font-size:90%;
}

#section-summary tr td {
    padding-right:150px;
    border-left-width: 0;
    border-bottom:1px solid #CCCCCC;
}

/*------------------------------------------------------*/

#section-results h2,
#section-qcresults h2 {
    padding-left:12px;
    margin-bottom:0;
    padding-bottom:0;
    margin-top:20px;
}
#section-results table,
#section-qcresults table {
    width:100%;
}
#section-results table thead tr,
#section-qcresults table thead tr {
    background-color:#DDDDDD;
}
#section-results table thead tr th,
#section-qcresults table thead tr th {
    padding-right:0;
}
#section-results table thead tr th span,
#section-qcresults table thead tr th span {
    display:block;
    border-bottom:1px solid #000000;
    line-height:1.6;
}

#section-results .analysis { width:200px; color: #000000; }
#section-results .previous{ width:250px; color:#555555; }
#section-results .result { width:100px; color: #000000; }
#section-results .specs { width:100px; text-align:center; color: #000000; }
#section-results .outofrange { width:10px; color: #ff0000; }
#section-results .remarks { font-size: 0.8em; padding: 0 650px 15px 3px; color: #555555; }

#section-qcresults .analysis {
    width:100px;
}

#section-qcresults .result {
    width:100px;
}

#section-qcresults .specs {
    width:100px;
    text-align:center;
}
#section-qcresults .worksheet {
    width:80px;
}
#section-qcresults .refsample {
    width:200px;
}
#section-qcresults .outofrange {
    width:10px;
}

/*---------------------------------------------*/

#section-attachments ul {
    list-style-type:none;
    padding:0;
    margin: 0 0 20px;
}

#section-attachments table#analysis-attachments td {
    padding-right:20px;
}

#section-signatures {
    margin-top:50px;
    width:100%
}
#section-signatures table {
    width:100%;
}

#section-signatures table tr td {
    text-align:center;
    padding-right:50px;
}

#section-discreeter {
    margin-top:60px;
    border-top:1px solid #AAAAAA;
    color:#444444;
}
#section-discreeter * {
    font-size:0.9em;
}
#section-discreeter ol {
    margin-left:0;
    padding-left:15px;
}
#footer {
    visibility:hidden;
    display:none;
}

@media print {
    * {
        font: 9pt Arial, Verdana, serif;
    }
    a {
        text-decoration:none;
        color:#000000;
    }

    #section-attachments {
        visibility:hidden;
        display:none;
    }
    .units {
        color:#555555;
        font-style:italic;
        font-size:10px;
    }

}
