* {
    font-family: Arial;
    font-size:12px;
}

html {
    background-color:#efefef;
}
html, body {
    margin: 0;
    padding: 0;
}
body {
    width: 210mm;
    padding: 10px 10px 0px 20px;
    margin: 10px auto;
    background-color:#fff;
}

h1 {
    font-size:1.4em;
    margin-top:40px;
    margin-bottom:10px;
    padding-bottom:5px;
    /*border-bottom: 1px solid #000;*/
}
h2 {
    font-size:1.2em;
}

.label {
    font-weight:bold;
}
.result {

}

.outofrange {
    font-weight:bold;
    color:#d40000;
}

.units {
    color:#777;
    font-weight:normal;
    font-style:italic;
    font-size:10px;
}

table tr td,
table tr th {
    text-align:left;
}

table#section-header{
    border-bottom:1px solid #000;
}

table#section-header,
table#section-info {
    width:100%;
    margin-bottom:20px;
}
table#section-header tr td,
table#section-info tr td {
    vertical-align:top;
}
table#section-header td#lab-logo {

}
table#section-header td#accreditation-logo {
    /*text-align:right;*/
    width:80mm;
}

table#section-info {
}
table#section-info td#client-info {
}
table#section-info td#lab-info {
    width:80mm;
}
#section-alert {
    padding:20px 20px 10px 20px;
    border:1px solid #CDCDCD;
    background-color:#efefef;
    margin:0px;
}
#section-alert h1 {
    font-size:1em;
    margin-top:0px;
    padding-bottom:0;
    border:none;
    color:#d40000;
}
#section-summary table,
#section-results table,
#section-qcresults table {
    padding-left:10px;
}

#section-summary tr td {
    padding-right:225px;
    border-left-width: 0px;
    border-bottom:1px solid #ccc;
}



#section-results h2,
#section-qcresults h2 {
    padding-left:13px;
    margin-bottom:0px;
    padding-bottom:0px;
    margin-top:20px;
}
#section-results table,
#section-qcresults table {
    width:100%;
}
#section-results table thead tr th,
#section-qcresults table thead tr th {
    padding-right:10px;
}
#section-results table thead tr th span,
#section-qcresults table thead tr th span {
    display:block;
    border-bottom:1px solid #000;
    line-height:1.6;
}

#section-results .result {
    width:100px;
}



#section-results .specs {
    width:200px;
    text-align:center;
    padding-right:10px;
}
#section-results .outofrange {
    width:10px;
}
#section-results .remarks {
    font-size: 0.9em;
    padding: 0 400px 15px 3px;
    color: #555;
}

#section-qcresults .result {
    width:60px;
}

#section-qcresults .specs {
    width:100px;
    text-align:center;
    padding-right:10px;
}
#section-qcresults .worksheet {
    width:80px;
}
#section-qcresults .refsample {
    width:200px;
}
#section-qcresults .outofrange {
    width:10px;
}

#section-attachments ul {
    list-style-type:none;
    margin:0px;
    padding:0px;
    margin-bottom:20px;
}

#section-attachments table#analysis-attachments td {
    padding-right:20px;
}

#section-signatures {
    margin-top:100px;
    width:100%
}
#section-signatures table {
    width:100%;
}

#section-signatures table tr td {
    text-align:center;
    padding-right:50px;
}

#section-discreeter {
    margin-top:60px;
    border-top:1px solid #AAA;
    color:#444;
}
#section-discreeter * {
    font-size:0.9em;
}
#section-discreeter ol {
    margin-left:0px;
    padding-left:15px;
}
#footer {
    visibility:hidden;
    display:none;
}


@media print {
    * {
        font: 9pt Arial, Verdana, serif;
    }
    a {
        text-decoration:none;
        color:#000;
    }
    .outofrange a {
        color:#d40000;
    }

    #section-attachments {
        visibility:hidden;
        display:none;
    }

}
