body {
    font-size: 12px;
}
h1 img {
    vertical-align: bottom;
    padding-right: 7px;
}
h1 {
    margin-bottom: 30px;
}
#chart {
    width: 100%;
    text-align: center;
    margin-top: 30px;
    margin-bottom: 30px;
}
div.chart-options label {
    padding-left: 10px;
}
div.chart-options {
    font-size: 13px;
}
div.chart-options select {
    border: 0 none;
    background: transparent;
    background-image: none;
    box-shadow: none;
    -webkit-appearance: none;
    display: inline-block;
    font-size: 1em;
    font-weight: bold;
}
div.chart-options a {
    display: none;
    visibility: hidden;
}
div.results-info {
    display: none;
    visibility: hidden;
}
table.bika-listing-table th {
    padding-bottom: 10px;
    font-size: 12px;
    text-align: left;
    border-bottom: 1px solid #6c6c6c;
    margin-bottom: 5px;
}
table.bika-listing-table {
    border: none;
    width: 100%;
    font-size: 12px;
}
table.bika-listing-table thead {
    padding-bottom: 10px;
}
table.bika-listing-table a {
    color: #000;
    text-decoration: none;
}
span.bold {
    font-weight:bold;
}

